<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Exception;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Session;
use Throwable;

class QueryFareInformativePriceAction
{
    use FileStorageTrait;

    private FareInformativePricingAction $fareInformativePricingRequest;
    private int $paxCount;

    public function __construct()
    {
        $this->fareInformativePricingRequest = new  FareInformativePricingAction;
    }

    /**
     * @throws Exception
     * @throws Throwable
     */
    public function execute(array $passenger, array $searchCriteriaFlight, array $segments, array $recommendation): array
    {
        $this->paxCount = count($passenger);
        $legs = count($searchCriteriaFlight);

        self::storeInFile("json", "1.1-selected-recommendation", json_encode($recommendation));
        self::storeInFile("json", "1.1-selected-segments", json_encode($segments));

        $isCorrectFareBasisReturned = false;
        $rbdIndex = 0;

        while ($isCorrectFareBasisReturned === false) {
            if ($rbdIndex > 0) {
                break;
            }

            Log::info('$rbdIndex: ' . $rbdIndex);
            $combinedSegments = [];

//            if ($legs === 1) {
//                $segments = [$segments];
//            }

            $otherCompany = $this->extractOtherCompany($recommendation);
            Session::put('other-company', $otherCompany);

            foreach ($segments as $index => $segment) {
                $groupOfFlight = $this->getGroupOfFlightWithBookingClass(
                    $segment,
                    $recommendation,
                    $index,
                    $legs === 1,
                    $rbdIndex
                );

                $combinedSegments[$index] = ['groupOfFlights' => $groupOfFlight];
            }

            self::storeInFile("json", "1.1-selected-combined-segments", json_encode($combinedSegments));
            Session::put('fare-informative-combined-segments', json_encode($combinedSegments));

            $fareInformativeReply = $this->fareInformativePricingRequest->execute($passenger, $combinedSegments, $otherCompany);

            if (isset($fareInformativeReply[0]['Fare_InformativePricingWithoutPNRReply']['errorGroup'])) {
                throw new Exception("An error occurred: " .
                    $fareInformativeReply[0]
                    ['Fare_InformativePricingWithoutPNRReply']['errorGroup']['errorWarningDescription']['freeText']
                );

//                Log::info("An error occurred: " . $fareInformativeReply[0]['Fare_InformativePricingWithoutPNRReply']['errorGroup']['errorWarningDescription']['freeText']);
//                continue;
            }

//            $isCorrectFareBasisReturned = $this->checkTheFareBasisOfFareInformativeReply($fareInformativeReply, $combinedSegments, $legs);
            $isCorrectFareBasisReturned = true;

            $rbdIndex++;
        }

        if ($isCorrectFareBasisReturned) {
            return $fareInformativeReply;
        }

        return [];
    }

    protected function getGroupOfFlightWithBookingClass(
        $groupOfFlight,
        $recommendation,
        $legIndex,
        bool $isOneWay,
        int $rbdIndex): array
    {
        if ($this->paxCount === 1) {
            $fareDetails = $recommendation['paxFareProduct']['fareDetails'];
        } else {
            $fareDetails = $recommendation['paxFareProduct'][0]['fareDetails'];
        }

        if ($isOneWay) {
            $groupOfFares = $fareDetails['groupOfFares'];
        } else {
            $groupOfFares = $fareDetails[$legIndex]['groupOfFares'];
        }

        if (isset($groupOfFlight['flightDetails']['flightInformation'])) {
            $productInformation = $groupOfFares['productInformation'];

            $rbd = $this->getRBDValue($productInformation, $rbdIndex);

            $fareBasis = $productInformation['fareProductDetail']['fareBasis'];

            $groupOfFlight['flightDetails']['flightInformation']['bookingClass'] = $rbd;
            $groupOfFlight['flightDetails']['flightInformation']['fareBasis'] = $fareBasis;
        } else {
            foreach ($groupOfFlight['flightDetails'] as $index => $flightDetails) {
                $productInformation = $groupOfFares[$index]['productInformation'];

                $rbd = $this->getRBDValue($productInformation, $rbdIndex);

                $fareBasis = $productInformation['fareProductDetail']['fareBasis'];

                $groupOfFlight['flightDetails'][$index]['flightInformation']['bookingClass'] = $rbd;
                $groupOfFlight['flightDetails'][$index]['flightInformation']['fareBasis'] = $fareBasis;
            }
        }

        return $groupOfFlight;
    }

//    protected function checkTheFareBasisOfFareInformativeReply(array $fareInformativeReply, $combinedSegments, int $legs): bool
//    {
//        if (!isset($fareInformativeReply[0]['Fare_InformativePricingWithoutPNRReply']['mainGroup'])) return false;
//
//        $fareComponentDetailsGroup = $fareInformativeReply[0]['Fare_InformativePricingWithoutPNRReply']['mainGroup']
//        ['pricingGroupLevelGroup']['fareInfoGroup']['fareComponentDetailsGroup'];
//
//        if ($legs === 1) {
//            $fareBasisInReply = $fareComponentDetailsGroup['componentClassInfo']['fareBasisDetails']['rateTariffClass'];
//            $fareBasisInSelectedSegments = $combinedSegments[0]['groupOfFlights']['flightDetails']['flightInformation']['fareBasis'];
//
//            return $fareBasisInReply === $fareBasisInSelectedSegments;
//        } else {
//            for ($legIndex = 0; $legIndex < $legs; $legIndex++) {
//                $fareBasisInReply = $fareComponentDetailsGroup[$legIndex]['componentClassInfo']['fareBasisDetails']['rateTariffClass'];
//                $fareBasisInSelectedSegments = $combinedSegments[$legIndex]['groupOfFlights']['flightDetails']['flightInformation']['fareBasis'];
//
//                if ($fareBasisInReply !== $fareBasisInSelectedSegments) {
//                    return false;
//                }
//            }
//        }
//
//        return true;
//    }

    protected function checkTheFareBasisOfFareInformativeReply(
        array $fareInformativeReply,
        $combinedSegments,
        int $legs
    ): bool
    {
        if (!isset($fareInformativeReply[0]['Fare_InformativePricingWithoutPNRReply']['mainGroup'])) return false;

        $fareComponentDetailsGroup = [];

        $pricingGroupLevelGroup = $fareInformativeReply[0]['Fare_InformativePricingWithoutPNRReply']['mainGroup']['pricingGroupLevelGroup'];

        if(isset($pricingGroupLevelGroup['fareInfoGroup'])) {
            $fareComponentDetailsGroup = $pricingGroupLevelGroup['fareInfoGroup']['fareComponentDetailsGroup'];
        } else {
            $fareComponentDetailsGroup = $pricingGroupLevelGroup[0]['fareInfoGroup']['fareComponentDetailsGroup'];
        }

        for ($legIndex = 0; $legIndex < $legs; $legIndex++) {
            $fareBasisInReply = ($legs === 1)
                ? $fareComponentDetailsGroup['componentClassInfo']['fareBasisDetails']['rateTariffClass']
                : $fareComponentDetailsGroup[$legIndex]['componentClassInfo']['fareBasisDetails']['rateTariffClass'];

            if (isset($combinedSegments[$legIndex]['groupOfFlights']['flightDetails']['flightInformation'])) {
                $fareBasisInSelectedSegments = $combinedSegments[$legIndex]['groupOfFlights']['flightDetails']['flightInformation']['fareBasis'];

                if ($fareBasisInReply !== $fareBasisInSelectedSegments) {
                    return false;
                }
            } else {
                foreach ($combinedSegments[$legIndex]['groupOfFlights']['flightDetails'] as $flightDetail) {
                    $fareBasisInSelectedSegments = $flightDetail['flightInformation']['fareBasis'];

                    if ($fareBasisInReply !== $fareBasisInSelectedSegments) {
                        return false;
                    }
                }
            }
        }

        return true;
    }


    protected function getRBDValue(mixed $productInformation, int $rbdIndex): string
    {
        $rbd = isset($productInformation['cabinProduct'][$rbdIndex])
            ? $productInformation['cabinProduct'][$rbdIndex]['rbd']
            : $productInformation['cabinProduct']['rbd'];

        Log::info('$rbd: '. $rbd);

        return $rbd;
    }

    protected function extractOtherCompany(array $recommendation): string
    {
        if ($this->paxCount === 1) {
            $paxFareProduct = $recommendation['paxFareProduct'];
        } else {
            $paxFareProduct = $recommendation['paxFareProduct'][0];
        }

        $codeShareDetails = $paxFareProduct['paxFareDetail']['codeShareDetails'];

        if(isset($codeShareDetails['transportStageQualifier'])) {
            return $codeShareDetails['company'];
        } else {
            $company = Collection::wrap($codeShareDetails)->where('transportStageQualifier',"V")->first();

            return $company['company'];
        }
    }

}
